@@ -17,7 +17,6 @@
 #include <sys/stat.h>
 #include <signal.h>
 #include <unistd.h>
-#include <wayland-client-protocol.h>
 #include <gbm.h>
 #include <fcntl.h>
 #include <xf86drm.h>
@@ -58,10 +57,72 @@ struct wf_recorder_output
     zxdg_output_v1 *zxdg_output;
     std::string name, description;
     int32_t x, y, width, height;
+    int32_t transform;
 };
 
 std::list<wf_recorder_output> available_outputs;
 
+static void
+display_handle_geometry(void *data,
+                        wl_output *,
+                        int32_t, int32_t,
+                        int32_t,
+                        int32_t,
+                        int32_t,
+                        const char *,
+                        const char *,
+                        int32_t transform)
+{
+    wf_recorder_output *wo = (wf_recorder_output*) data;
+
+    wo->transform = transform;
+}
+
+static void
+display_handle_mode(void *,
+                    struct wl_output *,
+                    uint32_t,
+                    int32_t,
+                    int32_t,
+                    int32_t)
+{
+}
+
+static void
+display_handle_done(void *, wl_output *)
+{
+}
+
+static void
+display_handle_scale(void *,
+                     wl_output *,
+                     int32_t)
+{
+}
+
+static void
+display_handle_name(void *,
+                    wl_output *,
+                    const char *)
+{
+}
+
+static void
+display_handle_description(void *,
+                           wl_output *,
+                           const char *)
+{
+}
+
+static const struct wl_output_listener output_listener = {
+	display_handle_geometry,
+	display_handle_mode,
+	display_handle_done,
+	display_handle_scale,
+	display_handle_name,
+	display_handle_description
+};
+
 static void handle_xdg_output_logical_position(void*,
     zxdg_output_v1* zxdg_output, int32_t x, int32_t y)
 {
@@ -466,10 +527,11 @@ static void handle_global(void*, struct wl_registry *registry,
 
     if (strcmp(interface, wl_output_interface.name) == 0)
     {
-        auto output = (wl_output*)wl_registry_bind(registry, name, &wl_output_interface, 1);
+        auto output = (wl_output*)wl_registry_bind(registry, name, &wl_output_interface, 4);
         wf_recorder_output wro;
         wro.output = output;
         available_outputs.push_back(wro);
+        wl_output_add_listener(output, &output_listener, &available_outputs.back());
     }
     else if (strcmp(interface, wl_shm_interface.name) == 0)
     {
@@ -611,6 +673,7 @@ static void write_loop(FrameWriterParams params)
         uint64_t sync_timestamp = 0;
         if (first_frame_ts.has_value()) {
             sync_timestamp = buffer.base_usec - first_frame_ts.value();
+#ifdef HAVE_AUDIO
         } else if (pr) {
             if (!pr->get_time_base() || pr->get_time_base() > buffer.base_usec) {
                 drop = true;
@@ -618,6 +681,7 @@ static void write_loop(FrameWriterParams params)
                 first_frame_ts = pr->get_time_base();
                 sync_timestamp = buffer.base_usec - first_frame_ts.value();
             }
+#endif
         } else {
             sync_timestamp = 0;
             first_frame_ts = buffer.base_usec;
@@ -728,7 +792,6 @@ static void sync_wayland()
     wl_display_roundtrip(display);
 }
 
-
 static void load_output_info()
 {
     for (auto& wo : available_outputs)
@@ -742,16 +805,23 @@ static void load_output_info()
     sync_wayland();
 }
 
-static wf_recorder_output* choose_interactive()
+static void print_available_outputs()
 {
-    fprintf(stdout, "Please select an output from the list to capture (enter output no.):\n");
-
     int i = 1;
     for (auto& wo : available_outputs)
     {
         printf("%d. Name: %s Description: %s\n", i++, wo.name.c_str(),
             wo.description.c_str());
     }
+}
+
+
+
+static wf_recorder_output* choose_interactive()
+{
+    fprintf(stdout, "Please select an output from the list to capture (enter output no.):\n");
+
+    print_available_outputs();
 
     printf("Enter output no.:");
     fflush(stdout);
@@ -879,6 +949,8 @@ Use Ctrl+C to stop.)");
 
   -l, --log                 Generates a log on the current terminal. Debug purposes.
 
+  -L, --list-output          List the available outputs.
+  
   -o, --output              Specify the output where the video is to be recorded.
 
   -p, --codec-param         Change the codec parameters.
@@ -987,6 +1059,29 @@ static void parse_codec_opts(std::map<std::string, std::string>& options, const
     }
 }
 
+static void init_wayland_client()
+{
+    display = wl_display_connect(NULL);
+    if (display == NULL)
+    {
+        fprintf(stderr, "failed to create display: %m\n");
+        exit(EXIT_FAILURE);
+    }
+    struct wl_registry *registry = wl_display_get_registry(display);
+    wl_registry_add_listener(registry, &registry_listener, NULL);
+    sync_wayland();
+}
+
+static void list_available_outputs()
+{
+    init_wayland_client();
+    load_output_info();
+    print_available_outputs();
+
+    exit(EXIT_SUCCESS);
+}
+
+
 int main(int argc, char *argv[])
 {
     FrameWriterParams params = FrameWriterParams(exit_main_loop);
@@ -1029,11 +1124,12 @@ int main(int argc, char *argv[])
         { "version",           no_argument,       NULL, 'v' },
         { "no-damage",         no_argument,       NULL, 'D' },
         { "overwrite",         no_argument,       NULL, 'y' },
+        { "list-output",       no_argument,       NULL, 'L' },
         { 0,                   0,                 NULL,  0  }
     };
 
     int c, i;
-    while((c = getopt_long(argc, argv, "o:f:m:g:c:p:r:x:C:P:R:X:d:b:B:la::hvDF:y", opts, &i)) != -1)
+    while((c = getopt_long(argc, argv, "o:f:m:g:c:p:r:x:C:P:R:X:d:b:B:la::hvDF:yL", opts, &i)) != -1)
     {
         switch(c)
         {
@@ -1135,10 +1231,14 @@ int main(int argc, char *argv[])
                 force_overwrite = true;
                 break;
 
+            case 'L':
+                list_available_outputs();
+                break;
+#ifdef HAVE_AUDIO
             case '*':
                 audioParams.audio_backend = optarg;
                 break;
-
+#endif
             default:
                 printf("Unsupported command line argument %s\n", optarg);
         }
@@ -1149,16 +1249,7 @@ int main(int argc, char *argv[])
         return EXIT_FAILURE;
     }
 
-    display = wl_display_connect(NULL);
-    if (display == NULL)
-    {
-        fprintf(stderr, "failed to create display: %m\n");
-        return EXIT_FAILURE;
-    }
-
-    struct wl_registry *registry = wl_display_get_registry(display);
-    wl_registry_add_listener(registry, &registry_listener, NULL);
-    sync_wayland();
+    init_wayland_client();
 
     if (params.codec.find("vaapi") != std::string::npos)
     {
@@ -1265,6 +1356,8 @@ int main(int argc, char *argv[])
         return EXIT_FAILURE;
     }
 
+    params.transform = chosen_output->transform;
+
     if (selected_region.is_selected())
     {
         if (!selected_region.contained_in({chosen_output->x, chosen_output->y,
@@ -1274,9 +1367,13 @@ int main(int argc, char *argv[])
                 "inside the output\n");
             selected_region = capture_region{};
         }
+    } else
+    {
+        selected_region = capture_region{chosen_output->x, chosen_output->y,
+            chosen_output->width, chosen_output->height};
     }
 
-    printf("selected region %d,%d %dx%d\n", selected_region.x, selected_region.y, selected_region.width, selected_region.height);
+    fprintf(stderr, "selected region %d,%d %dx%d\n", selected_region.x, selected_region.y, selected_region.width, selected_region.height);
 
     bool spawned_thread = false;
     std::thread writer_thread;
